package alg;

import java.io.*;

public class TreesArr {
static int [] L;
static int [] R;
static int [] Val;
static int [] size;
static int [] maxV;
static int [] minV;
static int [] Lpos;
static boolean [] isBST;
static int [] depth;

static int nil = -1;
static int treeSize = 0;
static int treeRoot = nil;

static void initTree(int tsize) {
  L = new int[tsize];
  R = new int[tsize];
  Val = new int[tsize];
  size = new int[tsize];
  maxV = new int[tsize];
  minV = new int[tsize];
  Lpos = new int[tsize];
  isBST = new boolean[tsize];
  depth = new int[tsize];

  for (int i = 0; i < tsize; i++)
    L[i] = R[i] = nil;
  treeRoot = -1; //  nil
}

static int setSizes(int root) {
  if (root == nil) return 0;
  size[root] = setSizes(L[root]) +1 + setSizes(R[root]);
  return size[root];
}

static int setMaxVals(int root) {
  if (root == nil) return Integer.MIN_VALUE;
  maxV[root] = Math.max(setMaxVals(L[root]), Val[root]);
  maxV[root] = Math.max(setMaxVals(R[root]), maxV[root]);
  return maxV[root];
}

static int setMinVals(int root) {
  if (root == nil) return Integer.MAX_VALUE;
  minV[root] = Math.min(setMinVals(L[root]), Val[root]);
  minV[root] = Math.min(setMinVals(R[root]), minV[root]);
  return minV[root];
}

static int setLpos(int root, int rootpos) {
  if (root == nil) return rootpos;
  Lpos[root] = setLpos(L[root], rootpos)+1;
  return setLpos(R[root], Lpos[root]);
}

static int height (int root) {
  if(root == nil) return -1;
  return 1+ Math.max(height(L[root]), height(R[root]));
}

static boolean setIsBST(int root){
  if (root == nil) return true;
  if (L[root] == nil)
    if (R[root] == nil)
      return true;  // leaf
    else
    isBST[root] = setIsBST(R[root]) && (Val[root] < minV[R[root]]);
  else
    if (R[root] == nil)
      isBST[root] = setIsBST(L[root]) && (maxV[L[root]] < Val[root] );
    else // both branches exist
      isBST[root] = (setIsBST(R[root]) & setIsBST(L[root])) // must call both
                    && (maxV[L[root]] < Val[root] ) && (Val[root] < minV[R[root]]) ;
  return isBST[root];
}

static int leftmostLeaf(int root) {
  if (root == nil) return nil; // for pathological cases
  if (L[root] != nil) return leftmostLeaf(L[root]);
  if (R[root] != nil) return leftmostLeaf(R[root]);
  return root;
}

static int findLeftmostMaxBST(int root, int bestSoFar) {
  if (root == nil) return bestSoFar;
  // if the node itself is a BST then there is no need to analyze its children
  if (isBST[root])
    if (( size[root] > size[bestSoFar]) ||
       ((size[root] == size[bestSoFar]) && (Lpos[root] < Lpos[bestSoFar] )))
      return root;
    else
      return bestSoFar;
  else {
    int bestFoundL, bestFoundR, bestFound;
    bestFoundL = findLeftmostMaxBST(L[root], bestSoFar);
    bestFoundR = findLeftmostMaxBST(R[root], bestSoFar);
    bestFound = (size[bestFoundL] < size[bestFoundR]) ? bestFoundR : bestFoundL;
    if ((size[bestFound] > size[bestSoFar]) ||
       ((size[bestFound] == size[bestSoFar]) && (Lpos[bestFound] < Lpos[bestSoFar] )) )
      return bestFound;
    else
      return bestSoFar;
  }
}


static int sgPos = 0; // single global variable keeps track of the position
                      // in the input string from which position the subsequent
                      // node/subtree is read and reconstructed in the memory.

static int buildTreeSg(String inString, int currDepth) {
       // returns the array index of the root of the subtree which is read from
       // the input string starting at the position sgPos declared above
       // the left and right subtrees are also recursively created in the process
 if (inString.charAt(sgPos) != '(' )
   return nil; // nothing to build
 int root;
 ++sgPos; // skip the opening bracket

 root = ++treeRoot;
 L[root] = buildTreeSg(inString, currDepth+1);
 Val[root] = 0;
 depth[root] = currDepth;
 while (('0' <= inString.charAt(sgPos) ) && (inString.charAt(sgPos) <= '9'))
   Val[root] = Val[root]*10 + (int) (inString.charAt(sgPos++)-48);
 R[root] = buildTreeSg(inString, currDepth+1);
 sgPos++;
 return root;
}

static int [] readArr(String inSg) {
  int arrsize = 1;
  for (int i = 0; i < inSg.length(); i++)
    if (inSg.charAt(i) == ' ') arrsize++;
  int [] arr = new int [arrsize];
  int val = 0;
  int ia = 0;
  for (int i = 0; i < inSg.length(); i++)
    if (inSg.charAt(i) != ' ')
      val = 10*val + (int)(inSg.charAt(i)-48);
    else {
      arr[ia++] = val;
      val = 0;
    }
  arr[ia] = val; // also the last
  return arr;
}

static int buildTreeArr(int [] arr1, int [] arr2, int low1, int high1, int low2, int high2, int currDepth) {
  if (low1 > high1) return nil;
  int root = ++treeRoot;
  Val[root] = arr2[low2];
  depth[root] = currDepth;
  int iroot1 = low1;
  while(arr1[iroot1] != Val[root])
    iroot1++;
  L[root] = buildTreeArr(arr1, arr2, low1, iroot1-1, low2+1, low2 + (iroot1-low1), currDepth+1);
  R[root] = buildTreeArr(arr1, arr2, iroot1+1, high1, low2 + (iroot1-low1) + 1, high2, currDepth+1);
  return root;
}

static void buildTreeFromLineExpr(String inString) {
  treeSize = 0;
  for(int i = 0; i < inString.length(); i++)
    if (inString.charAt(i) == '(' )
      treeSize++;
 initTree(treeSize);
 treeRoot = buildTreeSg(inString, 0);
}

static void buildTreeFromInPre(String inSg1, String inSg2) {
 int [] inArr1 = readArr(inSg1);
 int [] inArr2 = readArr(inSg2);
 treeSize = inArr1.length;
 initTree(treeSize);
 treeRoot =  buildTreeArr(inArr1, inArr2, 0, inArr1.length-1, 0, inArr2.length-1, 0);
}

public static void main(String[] args) throws IOException {

  BufferedReader br = new BufferedReader(new InputStreamReader(System.in));
  String inString1 = br.readLine();

  if (inString1.charAt(0) == '(')
    buildTreeFromLineExpr(inString1);
  else {
    String inString2 = br.readLine();
    buildTreeFromInPre(inString1, inString2);
  }

  setSizes(treeRoot);
  setMaxVals(treeRoot);
  setMinVals(treeRoot);
  setLpos(treeRoot, -1);
  setIsBST(treeRoot);

  int solutionNode = findLeftmostMaxBST(treeRoot, leftmostLeaf(treeRoot));

  System.out.printf("%d ", Val[solutionNode]);
  System.out.printf("%d ", size[solutionNode]);
  System.out.printf("%d ", depth[solutionNode]);
  System.out.printf("%d ", height(solutionNode));
  System.out.printf("%d\n", Lpos[solutionNode]);
}

}
