package alg;

import java.io.*;

class Node {
  Node left;
  Node right;
  int key;
  int size;     // number of nodes in the subtree of this node incl. this node
  int Lpos;     // position counted from the leftmost node of the tree
  int depth;
  int maxKey;   // max key value in the subtree of this node incl. this node
  int minKey;   // similarly
  boolean isBST;
  int height;
}

public class TreesObj {

static int setSizes(Node root) {
  if (root == null) return 0;
  root.size = setSizes(root.left) + 1 + setSizes(root.right);
  return root.size;
}

static int setMaxKeys(Node root) {
  if (root == null) return Integer.MIN_VALUE;
  root.maxKey = Math.max(setMaxKeys(root.left), root.key);
  root.maxKey = Math.max(root.maxKey, setMaxKeys(root.right));
  return root.maxKey;
}

static int setMinKeys(Node root) {
  if (root == null) return Integer.MAX_VALUE;
  root.minKey = Math.min(setMinKeys(root.left), root.key);
  root.minKey = Math.min(root.minKey, setMinKeys(root.right));
  return root.minKey;
}

static int setLpos(Node root, int rootpos ) {
  if (root == null) return rootpos;
  root.Lpos = setLpos(root.left, rootpos) + 1;
  return setLpos(root.right, root.Lpos);
}

static int depth(Node currNode, Node theNode, int currdepth ) {
  if (currNode == null) return -1;
  if (currNode == theNode) return currdepth;
  return Math.max(depth(currNode.left, theNode, currdepth+1),
                  depth(currNode.right, theNode, currdepth+1));
}

static int height (Node root) {
  if(root == null) return -1;
  return 1+ Math.max(height(root.left), height(root.right));
}

static boolean setIsBST(Node root) {
  if (root == null) return true;
  if ((root.left == null) && (root.right == null)) return true;
  if ((root.left == null))           // only right branch exists
    root.isBST = (setIsBST(root.right) && (root.key < root.right.minKey));
  else if ((root.right == null))    // only left branch exists
    root.isBST = (setIsBST(root.left) && (root.left.maxKey < root.key));
  else                              // both branches exist
    root.isBST =  (setIsBST(root.left) & (root.left.maxKey < root.key)
                 & setIsBST(root.right) & (root.key < root.right.minKey));
  return root.isBST;
}

static Node leftmostLeaf(Node root) {
  if (root.left != null)
    return leftmostLeaf(root.left);
  if (root.right != null)
    return leftmostLeaf(root.right);
  return root;
}

static Node findLeftmostMaxBSt(Node root, Node bestSoFar) {
  if (root == null) return bestSoFar;
  // if the node itself is a bst then there is no need to analyze its children
  if (root.isBST)
    if ((root.size > bestSoFar.size) ||
       ((root.size == bestSoFar.size) && (root.Lpos < bestSoFar.Lpos)))
        return root;
    else
      return bestSoFar;
  else { // root is not a root of BST
    Node bestFoundL = findLeftmostMaxBSt(root.left, bestSoFar);
    Node bestFoundR = findLeftmostMaxBSt(root.right, bestSoFar);
    Node bestFound = (bestFoundL.size < bestFoundR.size) ? bestFoundR: bestFoundL;
    if ((bestFound.size > bestSoFar.size) ||
       ((bestFound.size == bestSoFar.size) && (bestFound.Lpos < bestSoFar.Lpos)))
      return bestFound;
    else
      return bestSoFar;
  }
}

static int sgPos = 0; // single global variable to keep track of the position
                      // in the input string from which position the subsequent
                      // node/subtree is read and reconstructed in the memory.

static Node buildTreeFromLineExpr(String inString) {
       // returns the array index of the root of the subtree which is read from
       // the input string starting at the position sgPos declared above
       // the left and right subtrees are also recursively created in the process

  if (inString.charAt(sgPos) != '(') return null; //nothing to build
  ++ sgPos;     // skip the opening bracket
  Node L = buildTreeFromLineExpr(inString);
  Node root = new Node();
  root.left = L;
  root.key = 0;
  while (('0' <= inString.charAt(sgPos) ) && (inString.charAt(sgPos) <= '9'))
    root.key = root.key*10 + (int) (inString.charAt(sgPos++)-48);
  root.right = buildTreeFromLineExpr(inString);
 sgPos++;       // skip the closing bracket
 return root;
}

static int [] stringValsToArr(String inSg) {
  int arrsize = 1;
  for (int i = 0; i < inSg.length(); i++)
    if (inSg.charAt(i) == ' ') arrsize++;
  int [] arr = new int [arrsize];
  int val = 0;
  int ia = 0;
  for (int i = 0; i < inSg.length(); i++)
    if (inSg.charAt(i) != ' ')
      val = 10*val + (int)(inSg.charAt(i)-48);
    else {
      arr[ia++] = val;
      val = 0;
    }
  arr[ia] = val; // also the last
  return arr;
}

static Node buildTreeFromInPre(int [] arr1, int [] arr2, int low1, int high1, int low2, int high2) {
  if (low1 > high1)
    return null;

  Node root = new Node();
  root.key = arr2[low2];
  int iroot1 = low1;
  while(arr1[iroot1] != root.key)
    iroot1++;
  root.left = buildTreeFromInPre(arr1, arr2, low1, iroot1-1, low2+1, low2 + (iroot1-low1));
  root.right = buildTreeFromInPre(arr1, arr2, iroot1+1, high1, low2 + (iroot1-low1) + 1, high2);
  return root;
}

static void solution()  throws IOException {
  BufferedReader br = new BufferedReader(new InputStreamReader(System.in));
  String inString = br.readLine();

  Node treeRoot = null;
  if (inString.charAt(0) == '(')
     treeRoot = buildTreeFromLineExpr(inString);
  else {
    int [] arrInOrder = stringValsToArr(inString);
    inString = br.readLine();
    int [] arrPreOrder = stringValsToArr(inString);
    treeRoot = buildTreeFromInPre(arrInOrder, arrPreOrder,
                                  0, arrInOrder.length-1, 0, arrPreOrder.length-1);
  }

  setSizes(treeRoot);
  setMaxKeys(treeRoot);
  setMinKeys(treeRoot);
  setLpos(treeRoot, -1);
  setIsBST(treeRoot);

  Node solutionNode = findLeftmostMaxBSt(treeRoot, leftmostLeaf(treeRoot) );

  System.out.printf("%d ", solutionNode.key);
  System.out.printf("%d ", solutionNode.size);
  System.out.printf("%d ", depth(treeRoot, solutionNode, 0));
  System.out.printf("%d ", height(solutionNode));
  System.out.printf("%d\n", solutionNode.Lpos);
}

}
